package ginext

import (
	"errors"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/rs/zerolog/log"
)

type WHandlerFunc func(PreContext) HTTPResponse

func Wrap(w *GinWrapper, fn WHandlerFunc) gin.HandlerFunc {

	return func(g *gin.Context) {

		g.Set("__returnRawErrors", w.returnRawErrors)

		reqctx := g.Request.Context()

		wrap, stackTrace, panicObj := callPanicSafe(fn, PreContext{wrapper: w, ginCtx: g})
		if panicObj != nil {
			fmt.Printf("\n======== ======== STACKTRACE ======== ========\n%s\n======== ======== ======== ========\n\n", stackTrace)
			log.Error().
				Interface("panicObj", panicObj).
				Str("trace", stackTrace).
				Msg("Panic occured (in gin handler)")
			wrap = APIError(g, commonApiErr.Panic, "A panic occured in the HTTP handler", errors.New(fmt.Sprintf("%+v", panicObj)))
		}

		if g.Writer.Written() {
			panic("Writing in WrapperFunc is not supported")
		}

		if reqctx.Err() == nil {
			wrap.Write(g)
		}
	}
}
